// Copyright (c) 2013 by István Váradi

// This file is part of XPLRA, a remote-access plugin for X-Plane

// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:

// 1. Redistributions of source code must retain the above copyright notice, this
//    list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation
//    and/or other materials provided with the distribution.

// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
// ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

// The views and conclusions contained in the software and documentation are those
// of the authors and should not be interpreted as representing official policies,
// either expressed or implied, of the FreeBSD Project.

#ifndef HU_VARADIISTVAN_XPLRA_XPLANE_H
#define HU_VARADIISTVAN_XPLRA_XPLANE_H
//-----------------------------------------------------------------------------

#include "Exception.h"

#include <hu/varadiistvan/scpl/io/Waiter.h>

#include <inttypes.h>

//-----------------------------------------------------------------------------

namespace hu { namespace varadiistvan { namespace scpl { namespace io {

class LocalClientSocket;
class DataStream;

} /* namespace hu::varadiistvan::scpl::io */ } /* namespace hu::varadiistvan::scpl */ } /* namespace hu::varadiistvan */ } /* namespace hu */

//------------------------------------------------------------------------------

namespace hu { namespace varadiistvan { namespace xplra {

//------------------------------------------------------------------------------

/**
 * The main class to access X-Plane.
 *
 * The calls are synchronous and not thread-safe.
 */
class XPlane
{
private:
    /**
     * The waiter to use.
     */
    scpl::io::Waiter waiter;

    /**
     * The local client socket over which we are communicating with X-Plane.
     */
    scpl::io::LocalClientSocket* socket;

    /**
     * The data stream to handle the data conversions.
     */
    scpl::io::DataStream* stream;

public:
    /**
     * Construct the object. It will not be connected to the simulator
     * yet.
     */
    XPlane() throw();

    /**
     * Destroy the object. If connected, the connection will be
     * closed.
     */
    ~XPlane() throw();

    /**
     * Connect to the simulator.
     */
    void connect() throw(IOException);

    /**
     * Check if we are connected to the simulator.
     */
    bool isConnected() const throw();

    /**
     * Disconnect from the simulator.
     */
    void disconnect() throw();

    /**
     * Get the integer value of the dataref with the given name.
     */
    int getInt(const char* name) throw(Exception);

    /**
     * Get a float value.
     */
    float getFloat(const char* name) throw(Exception);

    /**
     * Get a double value.
     */
    double getDouble(const char* name) throw(Exception);

private:
    /**
     * Check if we have a stream and if it has not failed.
     */
    void checkStream() throw(NotConnectedException, IOException);

    /**
     * Check the given protocol result. If it signifies an error,
     * throw a ProtocolException with the correct error code.
     */
    void checkResult(uint8_t result) throw(ProtocolException);

    /**
     * Issue the command to get a single, scalar value. The result is
     * also checked, but the value should be read by the caller.
     */
    void getScalar(const char* name, uint8_t type) throw(Exception);
};

//------------------------------------------------------------------------------
// Inline definitions
//------------------------------------------------------------------------------

inline XPlane::XPlane() throw() :
    socket(0),
    stream(0)
{
}

//------------------------------------------------------------------------------

inline bool XPlane::isConnected() const throw()
{
    return socket!=0;
}

//------------------------------------------------------------------------------

} /* namespace hu::varadiistvan::xplra */ } /* namespace hu::varadiistvan */ } /* namespace hu */

//------------------------------------------------------------------------------
#endif // HU_VARADIISTVAN_XPLRA_XPLANE_H

// Local Variables:
// mode: C++
// c-basic-offset: 4
// indent-tabs-mode: nil
// End:
